using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace ColorMatrix
{
    public partial class Form1 : Form
    {
        Image imgPicture;
        Bitmap bmpPicture;
        System.Drawing.Imaging.ImageAttributes iaPicture;
        System.Drawing.Imaging.ColorMatrix cmPicture;
        Graphics gfxPicture;
        Rectangle rctPicture;

        public Form1()
        {
            InitializeComponent();
        }

        private void btnLoad_Click(object sender, EventArgs e)
        {
            // Open dialog to browse for picture
            if (openPicture.ShowDialog() == DialogResult.OK)
            {
                // Store the picture in an Image object from the file
                imgPicture = Image.FromFile(openPicture.FileName);
                // Set the picture in the PictureBox
                picPicture.Image = imgPicture;
                picPicture.Size = picPicture.Image.Size;
                // Set the size of the PictureBox and Form to fit the picture
                this.Size = picPicture.Image.Size;
                this.Width += 16;
                this.Height += 85;
            }
        }

        private void PreparePicture()
        {
            // If there's a picture
            if (imgPicture != null)
            {
                // Create new Bitmap object with the size of the picture
                bmpPicture = new Bitmap(imgPicture.Width, imgPicture.Height);
                // Image attributes for setting the attributes of the picture
                iaPicture = new System.Drawing.Imaging.ImageAttributes();
            }
        }

        private void FinalizePicture()
        {
            // Set the new color matrix
            iaPicture.SetColorMatrix(cmPicture);
            // Set the Graphics object from the bitmap
            gfxPicture = Graphics.FromImage(bmpPicture);
            // New rectangle for the picture, same size as the original picture
            rctPicture = new Rectangle(0, 0, imgPicture.Width, imgPicture.Height);
            // Draw the new image
            gfxPicture.DrawImage(imgPicture, rctPicture, 0, 0, imgPicture.Width, imgPicture.Height, GraphicsUnit.Pixel, iaPicture);
            // Set the PictureBox to the new inverted colors bitmap
            picPicture.Image = bmpPicture;
        }

        private void btnInvert_Click(object sender, EventArgs e)
        {
            PreparePicture();
            cmPicture = new System.Drawing.Imaging.ColorMatrix();
            // Change the elements
            cmPicture.Matrix00 = -1;
            cmPicture.Matrix11 = -1;
            cmPicture.Matrix22 = -1;
            FinalizePicture();
        }

        private void btnGrayscale_Click(object sender, EventArgs e)
        {
            PreparePicture();
            cmPicture = new System.Drawing.Imaging.ColorMatrix(new float[][]
            {
                new float[]{0.5f, 0.5f, 0.5f, 0, 0},
                new float[]{0.5f, 0.5f, 0.5f, 0, 0},
                new float[]{0.5f, 0.5f, 0.5f, 0, 0},
                new float[]{0, 0, 0, 1, 0, 0},
                new float[]{0, 0, 0, 0, 1, 0},
                new float[]{0, 0, 0, 0, 0, 1}
            });
            FinalizePicture();
        }

        private void btnTranslate_Click(object sender, EventArgs e)
        {
            PreparePicture();
            cmPicture = new System.Drawing.Imaging.ColorMatrix(new float[][]
            {
                new float[] {1, 0, 0, 0, 0},
                new float[] {0, 1, 0, 0, 0},
                new float[] {0, 0, 1, 0, 0},
                new float[] {0, 0, 0, 1, 0},
                new float[] {.0f, .50f, .0f, .0f, 1}
            });
            FinalizePicture();
        }
    }
}